//
// C++ Interface: kmfplugin
//
// Description: 
//
//
// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KMFPLUGIN_H
#define KMFPLUGIN_H

// TQt includes
class TQWidget;

#include <tqobject.h>

// KDE includes
#include <tdeparts/plugin.h>

// Project includes

namespace KMF {

class KMFNetwork;
class KMFRulesetDoc;
class KMFGenericDoc;
class KMFIPTDoc;
class KMFDoc;
class KMFError;

/**
@author Christian Hubinger
*/

class TDE_EXPORT KMFPlugin : public KParts::Plugin {
public:
	KMFPlugin( TQObject* parent , const char* name  );
    virtual ~KMFPlugin();
	void setOutputWidget( TQWidget* wid );
	void showEditor();
	void showOutput();
	void checkStatus();

protected:
	KMFRulesetDoc* rulesetDoc();
	KMFGenericDoc* genericDoc();
	KMFIPTDoc* iptablesDoc();
	KMFNetwork* network();
	KMFDoc* doc();
	KMFError* m_err;
};
}
#endif
